/*
 * LoadGrammar.java
 * Copyright (C) 2004-2006, Bjorn Bringert (bringert@cs.chalmers.se)
 * 
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * 
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
package se.chalmers.cs.gf.oaa;

import se.gu.ling.trindikit.oaa.common.*;
import com.sri.oaa2.icl.*;

import se.chalmers.cs.gf.GFException;
import se.chalmers.cs.gf.abssyn.*;
import se.chalmers.cs.gf.translate.*;
import se.chalmers.cs.gf.util.Pair;
import static se.chalmers.cs.gf.util.Pair.pair;

import java.util.*;

/**
 *  Implements the load_grammar solvable.
 */
public class LoadGrammar extends OAASolver {

        private static String solvableString = "load_grammar(PropFile,Name)";

        private Translators translators;

        public LoadGrammar(Translators translators) {
                super(solvableString);
                this.translators = translators;
        }


        public boolean solve(IclTerm goal, IclList params, IclList answers) {
	    String propFile;

	    try {
		propFile = ((IclStr)goal.getTerm(0)).toUnquotedString();
	    } catch (ClassCastException e){
		return false;
	    }

	    try {
		System.err.println("Loading grammar from " + propFile);

		Translator t = TranslatorFactory.createTranslator(propFile);
		translators.addTranslator(t);

		IclTerm ans = new IclStruct("load_grammar", 
					    new IclStr(propFile), 
					    new IclStr(t.getName()));
		answers.add(ans);
		return true;
	    } catch (Exception ex) {
		System.err.println(ex);
		return false;
	    }
	}

}
